import argparse
from wifind.learn import learn
from wifind.utils import clear
from wifind.predict import predict
from wifind.rooms import rooms

def main():
    try:
        parser = argparse.ArgumentParser(description="Scan Wi-Fi networks.")
        subparsers = parser.add_subparsers(title="subcommands", dest="subcommand")

        learn_parser = subparsers.add_parser("learn", help="Learn current position")
        learn_parser.add_argument("-r", "--room", help="Name of the room to sample", required=True)

        # depp_learn_parser = subparsers.add_parser("deep-learn", help="Learn current position")
        # depp_learn_parser.add_argument("-n", "--num", help="Number of samples to collect", required=True)
        # depp_learn_parser.add_argument("-r", "--room", help="Name of the room to sample", required=True)

        clean_parser = subparsers.add_parser("clear", help="Clear the data file")
        predict_parser = subparsers.add_parser("predict", help="Predict the room")
        
        room_parser = subparsers.add_parser("rooms", help="List the rooms in the data file")
        room_parser.add_argument("-s", "--samples", help="List the number of sample", required=False)

        args = parser.parse_args()

        if args.subcommand == "learn":
            learn(args.room)
        elif args.subcommand == "clear":
            clear()
        elif args.subcommand == "predict":
            predict()
        elif args.subcommand == "rooms":
            rooms(args.samples)
    except (KeyboardInterrupt, SystemExit):
        exit(0)

if __name__ == "__main__":
    main()


import os
import pickle
import json
import pandas as pd

from sklearn.ensemble import RandomForestClassifier
from sklearn.feature_extraction import DictVectorizer
from sklearn.pipeline import make_pipeline

DATA_PATH = "data.csv"
MODEL_PATH = "model.pkl"

def get_data():
    """
    This function is used to load the data from the file system.

    Returns:
        Dataframe: A dataframe.
    """
    if os.path.exists(DATA_PATH):
        return pd.read_csv(DATA_PATH, index_col=0)
    else:
        return pd.DataFrame()

def save_data(online_data):
    """
    This function is used to save the data to the file system.
    """
    # check file exists
    data = get_data()
    
    if data.empty:
        df = pd.DataFrame(online_data, index=[0])

        # Nan becomes 0
        df.fillna(0, inplace=True)
        df.to_csv(DATA_PATH)
    else:
        df = pd.concat([data, pd.DataFrame([online_data])], ignore_index=True)
        
        # Nan becomes 0
        df.fillna(0, inplace=True)
        df.to_csv(DATA_PATH)

def get_model():
    """
    This function is used to load the model from the file system.
    """
    with open(MODEL_PATH, 'rb') as file:  
        model = pickle.load(file)
        return model
    
def save_model(model):
    """
    This function is used to save the model to the file system.
    """
    with open(MODEL_PATH, "wb") as f:
        pickle.dump(model, f)

def get_pipeline(clf=RandomForestClassifier(n_estimators=100, class_weight="balanced")):
    return make_pipeline(DictVectorizer(sparse=False), clf)

def split_x(data: dict):
    return list(data.values())

def clear():
    """
    Clears data 
    """
    if os.path.exists(DATA_PATH):
        os.remove(DATA_PATH)


from sklearn.ensemble import RandomForestClassifier
from sklearn.naive_bayes import GaussianNB
from wifind.utils import get_data, get_pipeline, save_model, get_model

def train():
    """
    This function is used to train the model using the data in data.csv.
    """    
    data = get_data()
    model = get_model()

    X = data.drop("room", axis=1)
    y = data["room"]

    model.fit(X, y)

    save_model(model)


from wifind.utils import get_model
from wifind.utils import save_data, get_data
from wifind.scan import scan
import pandas as pd

def predict():
    # Load the model
    model = get_model()
    
    # Load the data
    data = get_data()
    
    # Get online data
    sample = scan(room=None)

    # Some APs may not be in the model, so we need to fill in the missing values
    X = pd.DataFrame([sample])

    # X has to contain all the columns in data, if X miss some columns it has to be added with the median value of the column
    missing_columns = set(data.drop(columns=["room"]).columns) - set(X.columns)
    for col in missing_columns:
        # Fill missing columns with the median value from the training data
        median_value = data[col].median()
        X[col] = median_value
        
    # X has to contain all the columns in data, if X has more columns than data, it has to be removed
    X = X[data.drop(columns=["room"]).columns]

    y = model.predict(X)
    y_prob = model.predict_proba(X)
    print(f'{y[0]}: {y_prob.max() * 100:.2f}%')


from wifind.utils import get_data

def rooms(samples=None):
    """
    This function is used to list the rooms in the data file.
    """
    data = get_data()
    ks = []
    for d in data:
        for key in d.keys():
            ks.append(key)
    
    # return the unique values
    print(list(set(ks)))


from access_points import get_scanner

def scan(room=None):
    """
    Scans for WiFi access points and returns a list of dictionaries containing the BSSID, SSID, signal quality, and room.

    Parameters:
    room (str): Optional; a string representing the room in which the scan is performed. Defaults to None.

    Returns: a list
    """
    wifi_scanner = get_scanner()
    aps = wifi_scanner.get_access_points()
    
    sample = {}
    for ap in aps:
        id = ap.get("bssid") + "-" + ap.get("ssid")    
        quality = ap.get("quality")
        sample[id] = quality
        if room:
            sample["room"] = room  

    return sample


from wifind.scan import scan
from wifind.utils import save_data 
from wifind.train import train

def learn(room):
    """
    This function is used to learn and gather data about a specific room.

    Parameters:
    room (str): The name of the room to gather data about.
    """
    # Obtain and store online data
    online_data = scan(room)
    save_data(online_data)
    train()